/************************************************************************
 *   IRC - Internet Relay Chat, m_core/m_clock.c
 *   This file is copyright (C) 2001 Andrew Suffield
 *                                    <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "ircd.h"
#include "common.h"
#include "numeric.h"
#include "irc_string.h"
#include "send.h"
#include "umodes.h"
#include "msg.h"
#include "handlers.h"
#include "channel.h"
#include "client.h"
#include "burst.h"
#include "s_log.h"

#include <string.h>

int m_clock(struct Client *, struct Client *, int, char **);

struct Message clock_msgtab = {
  "CLOCK", 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_ignore, m_clock},
  NULL, NULL
};

/*
 * m_clock
 * parv[0] == originator of lock request (bursting server)
 * parv[1] == channel to lock
 */
int
m_clock(struct Client *cptr, struct Client *sptr, int parc, char *parv[])
{
  struct Client *acptr;
  struct Channel *chptr;

  if (!local_burst)
    {
      if (burst_in_progress && (cptr != burst_in_progress))
	{
	  /* Drat. Something is bursting from another direction.
	   * Kill it, we cannot allow this
	   *
	   * This is the degenerate case where two of my peers
	   *  both picked up a client at the same time
	   *
	   * XXX: Most likely it should SQUIT the new server instead.
	   *
	   *  -- asuffield
	   */
	  sendto_ops_flag(UMODE_SEEROUTING,
			  "Burst is already in progress from %s, killing link",
			  burst_in_progress->name);
	  return exit_client(NULL, cptr, &me,
			     "Cannot run two bursts simultaneously");
	}
      else
	{
	  burst_in_progress = cptr;
	  sendto_local_ops_flag(UMODE_DEBUG,
				"%s is in a connect burst, finalising locks...",
				parv[0]);
	}
    }

  /* It's a query for a CLACK */
  if (waiting_for_clacks > 0)
    {
      sendto_ops_flag(UMODE_DEBUG,
		      "WTF: Received multiple CLACK requests, %s and %s, ignoring",
		      sptr->name, burst_in_progress->name);
      log(L_WARN, "WTF: Received multiple CLACK requests, %s and %s, ignoring",
	  sptr->name, burst_in_progress->name);
      return 0;
    }
  for (acptr = serv_cptr_list; acptr; acptr = acptr->next_server_client)
    {
      if (acptr->from == cptr)
	continue;
      if (acptr == &me) /* Don't think this can happen */
	continue;
      if (!acptr->serv)
	{
	  sendto_ops_flag(UMODE_DEBUG,
			  "WTF: Server %s has no serv structure",
			  acptr->name);
	  return 0;
	}
      acptr->serv->clacked = 0;
      waiting_for_clacks++;
      sendto_one(acptr, ":%s CLOCK", me.name);
    }
  if (!waiting_for_clacks)
    {
      /* I have no other servers (I'm a leaf), so CLACK away */
      sendto_one(sptr, ":%s CLACK", me.name);
      /* And remember it so that I don't try to burst
       * when my peer clacks me, because the peer will
       * be bursting at that point
       */
      already_clacked = 1;
    }

  return 0;

  return 0;
}
